# -*- coding: utf-8 -*-
"""
Admin
"""

from __future__ import unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from coop_cms.apps.rss_sync import models, views
from coop_cms.apps.rss_sync.forms import RssSourceAdminForm, RssItemAdminForm


class RssSourceAdmin(admin.ModelAdmin):
    """Rss source admin"""

    list_display = ('url', 'title', 'last_collect')
    readonly_fields = ('title', 'last_collect')
    fieldsets = (
        (_('Feed'), {'fields': ('url',)}),
        (_('Information'), {'fields': ('title', 'last_collect')}),
    )
    actions = [views.collect_rss_items_action]

    def get_form(self, request, obj=None, **kwargs):
        """returns custom form"""
        defaults = dict(kwargs)
        if obj:
            defaults.update({
                'form': RssSourceAdminForm,
            })
        return super(RssSourceAdmin, self).get_form(request, obj, **defaults) # pylint: disable=E1002

admin.site.register(models.RssSource, RssSourceAdmin)


class RssItemAdmin(admin.ModelAdmin):
    """Rss Item admin"""
    form = RssItemAdminForm
    list_display = ('link', 'source', 'title', 'updated', 'processed')
    list_filter = ('source', 'processed')
    readonly_fields = ('source', 'link', 'updated', 'processed')
    fieldsets = (
        (_('Actions'), {'fields': ('id',)}),
        (_('Source'), {'fields': ('source', 'link', 'updated')}),
        (_('Content'), {'fields': ('title', 'summary')}),
        (_('CMS'), {'fields': ('processed',)}),
    )
    actions = [views.create_cms_article_action]

admin.site.register(models.RssItem, RssItemAdmin)



# -*- coding: utf-8 -*-
"""
Unit tests
"""

from __future__ import unicode_literals

from unittest import skipUnless

from django.contrib.auth.models import User
from django.conf import settings
from django.test import TestCase

from model_mommy import mommy

from coop_cms.settings import get_article_class


@skipUnless('coop_cms.apps.demo_cms' in settings.INSTALLED_APPS, "demo_cms not installed installed")
class AuthorPermissionTest(TestCase):
    """test author permission is taken into account"""

    def setUp(self):
        """called before every test"""
        if hasattr(get_article_class, '_cache_class'):
            delattr(get_article_class, '_cache_class')
        settings.COOP_CMS_ARTICLE_CLASS = 'coop_cms.apps.demo_cms.models.PrivateArticle'
        self.user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')

    def tearDown(self):
        """called after every test"""
        delattr(get_article_class, '_cache_class')
        settings.COOP_CMS_ARTICLE_CLASS = 'coop_cms.apps.demo_cms.models.Article'

    def test_view_private_article(self):
        """test user can view his private article"""
        article = mommy.make(get_article_class(), author=self.user)
        self.assertTrue(self.client.login(username=self.user.username, password='toto'))
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)

    def test_cant_view_private_article(self):
        """test user can not view other user private article"""
        article = mommy.make(get_article_class())

        response = self.client.get(article.get_absolute_url())
        self.assertEqual(404, response.status_code)

        self.assertTrue(self.client.login(username=self.user.username, password='toto'))
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(404, response.status_code)

    def test_edit_private_article(self):
        """test user can edit own private article"""
        article = mommy.make(get_article_class(), author=self.user)
        self.assertTrue(self.client.login(username=self.user.username, password='toto'))
        data = {'title': 'A', 'content': 'B', 'author': article.author.id}
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(200, response.status_code)
        article = get_article_class().objects.get(id=article.id)#refresh
        self.assertEqual('A', article.title)
        self.assertEqual('B', article.content)

    def test_cant_edit_private_article(self):
        """test user can not edit other user article"""
        klass = get_article_class()
        article = mommy.make(klass, publication=klass.DRAFT)
        self.assertTrue(self.client.login(username=self.user.username, password='toto'))
        data = {'title': 'A', 'content': 'B', 'author': None}
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(403, response.status_code)
        article = get_article_class().objects.get(id=article.id)#refresh
        self.assertNotEqual('A', article.title)
        self.assertNotEqual('B', article.content)

    def test_publish_private_article(self):
        """test user can publish own private article"""
        klass = get_article_class()
        article = mommy.make(klass, author=self.user, publication=klass.DRAFT)
        self.assertTrue(self.client.login(username=self.user.username, password='toto'))
        response = self.client.post(article.get_publish_url(), data={'publication':klass.PUBLISHED}, follow=True)
        self.assertEqual(200, response.status_code)
        article = klass.objects.get(id=article.id)#refresh
        self.assertEqual(article.publication, klass.PUBLISHED)

    def test_cant_publish_private_article(self):
        """test user can not publish others private article"""
        klass = get_article_class()
        article = mommy.make(klass, publication=klass.DRAFT)
        self.assertTrue(self.client.login(username=self.user.username, password='toto'))
        response = self.client.post(article.get_publish_url(), data={'publication':klass.PUBLISHED}, follow=True)
        self.assertEqual(403, response.status_code)
        article = klass.objects.get(id=article.id)#refresh
        self.assertEqual(article.publication, klass.DRAFT)

    def test_can_change_author_article(self):
        """test user can change author"""
        klass = get_article_class()
        article = mommy.make(klass, author=self.user, publication=klass.PUBLISHED)

        titi = User.objects.create_user('titi', 'titi@toto.fr', 'toto')

        self.assertTrue(self.client.login(username=self.user.username, password='toto'))
        data = {'title': 'A', 'content': 'B', 'author': titi.id}
        response = self.client.post(article.get_edit_url(), data=data, follow=False)
        self.assertEqual(302, response.status_code)

        article = klass.objects.get(id=article.id)  # refresh
        self.assertEqual(article.author, titi)



# -*- coding: utf-8 -*-
"""forms"""

from __future__ import unicode_literals

import floppyforms.__future__ as floppyforms

from coop_html_editor.widgets import get_inline_html_widget


class InlineHtmlEditableModelForm(floppyforms.ModelForm):
    """Base class for form with inline-HTML editor fields"""
    is_inline_editable = True  # The cms_edition templatetag checks this for switching to edit mode

    def __init__(self, *args, **kwargs):
        super(InlineHtmlEditableModelForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        for field_name in self.Meta.fields:
            no_inline_html_widgets = getattr(self.Meta, 'no_inline_editable_widgets', ())
            if field_name not in no_inline_html_widgets:
                self.fields[field_name].widget = get_inline_html_widget()

    class Media:
        css = {
            'all': ('css/colorbox.css', ),
        }
        js = (
            'js/jquery.form.js',
            'js/jquery.pageslide.js',
            'js/jquery.colorbox-min.js',
            'js/colorbox.coop.js',
        )



# -*- coding: utf-8 -*-
"""Use email rather than username for user authentication"""

from django.contrib import admin

from .models import InvalidatedUser


class InvalidatedUserAdmin(admin.ModelAdmin):
    list_display = ('user', 'password_changed', )
    list_filter = ('password_changed', )
    date_hierarchy = 'invalidation_datetime'
    search_fields = ('user__email', )

admin.site.register(InvalidatedUser, InvalidatedUserAdmin)



# -*- coding: utf-8 -*-
"""forms"""

from __future__ import unicode_literals

from django import forms
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _

from coop_cms.forms.base import InlineHtmlEditableModelForm
from coop_cms.forms.navigation import WithNavigationModelForm
from coop_cms.models import BaseArticle
from coop_cms.settings import (
    get_article_class, get_article_templates, is_localized, can_rewrite_url, is_multi_site
)
from coop_cms.utils import dehtml
from coop_cms.widgets import ImageEdit, ReadOnlyInput


class ArticleForm(InlineHtmlEditableModelForm):
    """frontend edition of an article"""

    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002

        self.article = kwargs.get('instance', None)
        self.set_logo_size()
        if getattr(settings, 'COOP_CMS_TITLE_OPTIONAL', False):
            # Optional title : make possible to remove the title from a template
            self.fields['title'].required = False

    class Meta:
        model = get_article_class()
        fields = ('title', 'subtitle', 'content', 'logo')
        no_inline_html_widgets = ('logo',)

    def set_logo_size(self, logo_size=None, logo_crop=None):
        """change logo size"""
        if 'logo' in self.fields:
            thumbnail_src = self.logo_thumbnail(logo_size, logo_crop)
            update_url = reverse('coop_cms_update_logo', args=[self.article.id])
            self.fields['logo'].widget = ImageEdit(
                update_url,
                thumbnail_src.url if thumbnail_src else '',
                attrs={"class": "resizable"}
            )

    def logo_thumbnail(self, logo_size=None, logo_crop=None):
        """transform logo into thumbnail"""
        if self.article:
            return self.article.logo_thumbnail(True, logo_size=logo_size, logo_crop=logo_crop)

    def clean_title(self):
        """article title validation"""
        if getattr(settings, 'COOP_CMS_TITLE_OPTIONAL', False):
            title = self.cleaned_data['title']
            if not title and self.article:
                # if the title is optional and nothing is set
                # We do not modify it when saving
                return self.article.title
        else:
            title = self.cleaned_data['title'].strip()
            if title[-4:].lower() == '<br>':
                title = title[:-4]
            if not title:
                raise ValidationError(_("Title can not be empty"))
        return title


class BaseArticleAdminForm(forms.ModelForm):
    """base form for article admin"""

    def __init__(self, *args, **kwargs):
        super(BaseArticleAdminForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        self.article = kwargs.get('instance', None)
        templates = get_article_templates(self.article, getattr(self, "current_user", None))
        if templates:
            self.fields['template'].widget = forms.Select(choices=templates)

        self.slug_fields = []
        if is_localized():
            for lang_and_name in settings.LANGUAGES:
                from modeltranslation.utils import build_localized_fieldname
                field_name = build_localized_fieldname('slug', lang_and_name[0])
                self.slug_fields.append(field_name)
        else:
            self.slug_fields = ['slug']

        can_change_article_slug = can_rewrite_url()

        if not can_change_article_slug:
            can_change_article_slug = (self.article.publication != BaseArticle.PUBLISHED) if self.article else True

        for slug_field in self.slug_fields:
            if not can_change_article_slug:
                self.fields[slug_field].widget = ReadOnlyInput()


class ArticleAdminForm(BaseArticleAdminForm):
    """admin form for article"""

    class Meta:
        model = get_article_class()
        fields = (
            'slug', 'title', 'subtitle', 'content', 'template', 'publication', 'logo', 'temp_logo',
            'summary', 'category', 'in_newsletter', 'homepage_for_site', 'headline', 'publication_date', 'sites',
            'login_required',
        )
        widgets = {
            'title': forms.TextInput(attrs={'size': 100})
        }


class ArticleTemplateForm(forms.Form):
    """article template form"""

    def __init__(self, article, user, *args, **kwargs):
        super(ArticleTemplateForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        choices = get_article_templates(article, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["template"].initial = article.template


class ArticleLogoForm(forms.Form):
    """article logo form"""
    image = forms.ImageField(required=True, label=_('Logo'),)


class ArticleSettingsForm(WithNavigationModelForm):
    """article settings"""
    class Meta:
        model = get_article_class()
        fields = (
            'template', 'category', 'publication', 'publication_date', 'headline', 'in_newsletter', 'summary', 'sites',
            'login_required',
        )
        widgets = {
            'sites': forms.CheckboxSelectMultiple(),
        }

    def __init__(self, user, *args, **kwargs):
        article = kwargs['instance']

        try:
            initials = kwargs['initial']
        except KeyError:
            initials = {}
        summary = article.summary
        if not summary:
            summary = dehtml(article.content)[:400]
        initials.update({'summary': summary})
        initials.update({'publication_date': article.publication_date.strftime("%Y-%m-%d %H:%M:%S")})
        
        kwargs['initial'] = initials
        super(ArticleSettingsForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002

        self.fields['category'].queryset = self.fields['category'].queryset.filter(sites=settings.SITE_ID)

        choices = get_article_templates(article, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()

        if 'sites' in self.fields and not is_multi_site():
            self.fields['sites'].widget = forms.MultipleHiddenInput()


class NewArticleForm(WithNavigationModelForm):
    """New article form"""
    class Meta:
        model = get_article_class()
        fields = (
            'title', 'template', 'category', 'headline', 'publication', 'in_newsletter', 'sites', 'login_required',
        )
        widgets = {
            'sites': forms.CheckboxSelectMultiple(),
        }

    def __init__(self, user, *args, **kwargs):
        super(NewArticleForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        choices = get_article_templates(None, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["title"].required = True
        self.fields["title"].widget = forms.TextInput(attrs={'size': 30})

        self.fields['category'].queryset = self.fields['category'].queryset.filter(sites=settings.SITE_ID)

        if 'sites' in self.fields:
            self.fields['sites'].initial = self.get_initials('sites')
            if not is_multi_site():
                self.fields['sites'].widget = forms.MultipleHiddenInput()

    def get_initials(self, field_name):
        """return the initial values"""
        if field_name == 'sites':
            return [Site.objects.get_current()]

    def clean_site(self):
        """check that the current site is selected"""
        sites = self.cleaned_data['sites']
        if Site.objects.get_current() not in sites:
            raise ValidationError(_("It is recommended to keep the current site."))
        return sites


class PublishArticleForm(forms.ModelForm):
    """Publish article form"""
    class Meta:
        model = get_article_class()
        fields = ('publication',)
        widgets = {
            'publication': forms.HiddenInput(),
        }



# -*- coding: utf-8 -*-

# This can be imported directly from coop_cms.forms
from __future__ import unicode_literals

from coop_cms.forms.articles import (
    ArticleForm, ArticleAdminForm, ArticleSettingsForm, NewArticleForm, BaseArticleAdminForm
)
from coop_cms.forms.newsletters import NewsletterForm, NewsletterSettingsForm

__all__ = [
    'ArticleForm', 'ArticleAdminForm', 'ArticleSettingsForm', 'BaseArticleAdminForm', 'NewArticleForm',
    'NewsletterForm', 'NewsletterSettingsForm'
]



# -*- coding: utf-8 -*-
"""forms"""

from __future__ import unicode_literals

import floppyforms.__future__ as forms

from coop_html_editor.widgets import get_inline_html_widget

from coop_cms.apps.test_app.models import TestClass
from coop_cms.forms.articles import NewArticleForm, ArticleSettingsForm
from coop_cms.forms.base import InlineHtmlEditableModelForm
from coop_cms.forms.newsletters import NewsletterSettingsForm


class TestClassForm(InlineHtmlEditableModelForm):
    """for unit-testing"""
    class Meta:
        model = TestClass
        fields = ('field1', 'field2', 'field3', 'bool_field', 'int_field', 'float_field')
        widgets = {
            'field2': get_inline_html_widget(),
        }
        no_inline_html_widgets = ('field2', 'field3', 'bool_field', 'int_field', 'float_field')


class MyNewArticleForm(NewArticleForm):
    """for unit-testing"""
    dummy = forms.CharField(required=False)


class MyArticleSettingsForm(ArticleSettingsForm):
    """for unit-testing"""
    dummy = forms.CharField(required=False)


class MyNewsletterSettingsForm(NewsletterSettingsForm):
    """for unit-testing"""
    dummy = forms.CharField(required=False)



# -*- coding: utf-8 -*-
"""utils"""

from __future__ import unicode_literals

from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger


def paginate(request, queryset, items_count):
    try:
        page = int(request.GET.get('page', 0) or 0)
    except ValueError:
        page = 1
    paginator = Paginator(queryset, items_count)
    try:
        page_obj = paginator.page(page or 1)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page
        page_obj = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        page_obj = paginator.page(paginator.num_pages)
    return page_obj



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-08 16:00
from __future__ import unicode_literals

from django.db import migrations, models
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0014_auto_20171006_1040'),
    ]

    operations = [
        migrations.AlterField(
            model_name='alias',
            name='redirect_url',
            field=models.CharField(blank=True, default='', max_length=200),
        ),
        migrations.AlterField(
            model_name='articlecategory',
            name='slug',
            field=django_extensions.db.fields.AutoSlugField(blank=True, editable=False, max_length=100, populate_from='name', unique=True),
        ),
        migrations.AlterField(
            model_name='document',
            name='name',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='fragmenttype',
            name='allowed_css_classes',
            field=models.CharField(default='', help_text='the css classed proposed when editing a fragment. It must be separated by comas', max_length=200, verbose_name='allowed css classes'),
        ),
        migrations.AlterField(
            model_name='image',
            name='copyright',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='copyright'),
        ),
        migrations.AlterField(
            model_name='image',
            name='name',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='imagesize',
            name='crop',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='crop'),
        ),
        migrations.AlterField(
            model_name='navtree',
            name='name',
            field=models.CharField(db_index=True, default='default', max_length=100, unique=True, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='navtype',
            name='search_field',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='search field'),
        ),
        migrations.AlterField(
            model_name='newsletter',
            name='content',
            field=models.TextField(blank=True, default='<br>', verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='newsletter',
            name='source_url',
            field=models.URLField(blank=True, default='', verbose_name='source url'),
        ),
        migrations.AlterField(
            model_name='newsletter',
            name='subject',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='subject'),
        ),
        migrations.AlterField(
            model_name='newsletter',
            name='template',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='template'),
        ),
        migrations.AlterField(
            model_name='pieceofhtml',
            name='content',
            field=models.TextField(blank=True, default='', verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='pieceofhtml',
            name='extra_id',
            field=models.CharField(blank=True, db_index=True, default='', max_length=100, verbose_name='extra identifier'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Applicaion for unit testing"""

from __future__ import unicode_literals

from django.apps import AppConfig

class CoopCmsTestAppConfig(AppConfig):
    name = 'coop_cms.apps.test_app'
    verbose_name = "coop CMS > App for unit test"



# -*- coding: utf-8 -*-

# Borrowed from https://bitbucket.org/wkornewald/django-filetransfers

import mimetypes

from django.conf import settings

from django.http import HttpResponse
from django.utils.encoding import smart_str


class ChunkedFile(object):
    def __init__(self, file):
        self.file = file

    def __iter__(self):
        return self.file.chunks()


def chunk_serve_file(request, file, save_as, content_type, **kwargs):
    """
    Serves the file in chunks for efficiency reasons, but the transfer still
    goes through Django itself, so it's much worse than using the web server,
    but at least it works with all configurations.
    """
    response = HttpResponse(ChunkedFile(file), content_type=content_type)
    if save_as:
        response['Content-Disposition'] = smart_str(u'attachment; filename={0}'.format(save_as))
    if file.size is not None:
        response['Content-Length'] = file.size
    return response


def xsendfile_serve_file(request, file, save_as, content_type, **kwargs):
    """Lets the web server serve the file using the X-Sendfile extension"""
    response = HttpResponse(content_type=content_type)
    file_path = file.name
    response['X-Sendfile'] = smart_str(file_path)
    if save_as:
        response['Content-Disposition'] = smart_str(u'attachment; filename={0}'.format(save_as))
    if file.size is not None:
        response['Content-Length'] = file.size
    return response


def serve_file(request, file, backend=None, save_as=False, content_type=None):
    # Backends are responsible for handling range requests.
    filename = file.name.rsplit('/')[-1]
    if save_as is True:
        save_as = filename
    if not content_type:
        content_type = mimetypes.guess_type(filename)[0]

    if settings.DEBUG:
        return chunk_serve_file(request, file, save_as=save_as, content_type=content_type)
    else:
        return xsendfile_serve_file(request, file, save_as=save_as, content_type=content_type)


# -*- coding: utf-8 -*-
"""for unit-testing"""

from __future__ import unicode_literals

from coop_cms.apps.test_app.models import TestClass
from coop_cms.apps.test_app.forms import TestClassForm

from coop_cms.generic_views import EditableObjectView, ListView, EditableFormsetView


class TestClassListView(ListView):
    """for unit-testing"""
    model = TestClass
    template_name = "coop_cms/test_app/list.html"
    ordering = 'other_field'


class TestClassDetailView(EditableObjectView):
    """for unit-testing"""
    model = TestClass
    edit_mode = False
    form_class = TestClassForm
    template_name = "coop_cms/test_app/detail.html"


class TestClassEditView(TestClassDetailView):
    """for unit-testing"""
    edit_mode = True


class TestClassFormsetView(EditableFormsetView):
    """for unit-testing"""
    model = TestClass
    edit_mode = False
    form_class = TestClassForm
    template_name = "coop_cms/test_app/formset.html"
    success_view_name = 'coop_cms_testapp_formset'


class TestClassFormsetEditView(TestClassFormsetView):
    """for unit-testing"""
    edit_mode = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0007_newsletter_newsletter_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='alias',
            name='redirect_code',
            field=models.IntegerField(default=301, choices=[(301, '301 - Permanent'), (302, '302 - Non permanent')]),
        ),
    ]



# -*- coding: utf-8 -*-
"""models"""

from django.contrib.auth.models import User
from django.db import models


class InvalidatedUser(models.Model):
    """Show a message for these users"""
    user = models.ForeignKey(User)
    password_changed = models.BooleanField(default=False)
    invalidation_datetime = models.DateTimeField()

    def __unicode__(self):
        return u'{0}'.format(self.user.email)



# -*- coding: utf-8 -*-
"""
Some tools for templates
"""

from __future__ import unicode_literals

from django import template

from coop_cms.templatetags.coop_utils import is_checkbox as _is_checkbox
from coop_cms.templatetags.coop_navigation import NavigationAsNestedUlNode, extract_kwargs

register = template.Library()

# Just for compatibility

@register.filter(name='is_checkbox')
def is_checkbox(field):
    """returns true if field is a checkbox"""
    return _is_checkbox(field)


@register.tag
def navigation_bootstrap(parser, token):
    """returns the bootstrap-friendly navigation"""
    kwargs = dict(li_node="coop_bootstrap/li_node.html")
    args = token.contents.split()
    kwargs.update(extract_kwargs(args))
    return NavigationAsNestedUlNode(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0012_auto_20170502_1125'),
    ]

    operations = [

    ]



# -*- coding: utf-8 -*-

from datetime import datetime

from django.core.management.base import BaseCommand
from django.contrib.auth.models import User
from django.utils.crypto import get_random_string

from ...models import InvalidatedUser


class Command(BaseCommand):
    help = u"send waiting messages by emails to the doctors"

    def handle(self, *args, **options):
        # look for emailing to be sent
        for user in User.objects.all():
            password = get_random_string(length=12)
            user.set_password(password)
            user.save()

            InvalidatedUser.objects.create(
                user=user,
                invalidation_datetime=datetime.now()
            )



# -*- coding: utf-8 -*-
"""
Example of a custom coop_bar configuration
"""
from __future__ import unicode_literals

from coop_cms import coop_bar_cfg


def load_commands(coop_bar):
    """This is where to define menus displayed by the coop_bar"""

    coop_bar.register([
        [
            coop_bar_cfg.change_newsletter_settings,
            coop_bar_cfg.test_newsletter,
        ],
        [
            coop_bar_cfg.cms_edit,
            coop_bar_cfg.cms_save,
            coop_bar_cfg.cms_cancel],
        [
            coop_bar_cfg.cms_new_article,
            coop_bar_cfg.cms_article_settings,
        ],
        [
            coop_bar_cfg.cms_publish,
        ],
        [
            coop_bar_cfg.cms_media_library,
            coop_bar_cfg.cms_upload_image,
            coop_bar_cfg.cms_upload_doc,
        ],
        [
            coop_bar_cfg.log_out,
        ],
    ])




# -*- coding: utf-8 -*-
"""custom backend for sorl-thumbnail: keep image file name"""

from __future__ import unicode_literals

import os.path

from sorl.thumbnail.base import ThumbnailBackend, EXTENSIONS
from sorl.thumbnail.conf import settings
from sorl.thumbnail.helpers import tokey, serialize


# activated by adding following line to the settings.py
# THUMBNAIL_BACKEND = 'coop_cms.thumbnail_backend.KeepNameThumbnailBackend'
# This backend keeps the filename

class KeepNameThumbnailBackend(ThumbnailBackend):
    """This backend keep the filename when generating a thumbnail image"""
    
    def _get_thumbnail_filename(self, source, geometry_string, options):
        """Computes the destination filename."""

        key = tokey(source.key, geometry_string, serialize(options))
        
        filename, _ext = os.path.splitext(os.path.basename(source.name))
        
        path = '{0}/{1}'.format(key, filename)
        image_format = options['format']

        return '{0}{1}.{2}'.format(settings.THUMBNAIL_PREFIX, path, EXTENSIONS[image_format])



# -*- coding: utf-8 -*-
"""forms"""

from __future__ import unicode_literals

from six import string_types

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

import floppyforms.__future__ as floppyforms

from coop_cms.models import Fragment, FragmentType


class BaseFragmentForm(floppyforms.ModelForm):
    """Base class for AddFragmentForm and EditFragmentForm"""

    class Media:
        css = {
            'all': ('chosen/chosen.css', ),
        }
        js = (
            'chosen/chosen.jquery.js',
        )

    def post_init(self, all_classes=False):
        """init"""
        try:
            instance_fragment_type = self.instance.type
        except FragmentType.DoesNotExist:
            instance_fragment_type = None

        if instance_fragment_type or all_classes:
            if instance_fragment_type:
                css_classes = instance_fragment_type.allowed_css_classes.split(',')
            else:
                css_classes = []
                for fragment_type in FragmentType.objects.all():
                    fragment_classes = fragment_type.allowed_css_classes.split(',')
                    for css_class in fragment_classes:
                        if css_class not in css_classes:
                            css_classes.append(css_class)
            if css_classes:
                choices = [('', '')] + [(x, x) for x in css_classes]
                self.fields['css_class'].widget = floppyforms.SelectMultiple(
                    choices=choices,
                    attrs={"class": "chosen-select", "data-placeholder": _("Select CSS classes to apply")}
                )
            else:
                self.fields['css_class'].widget = floppyforms.HiddenInput()
        else:
            self.fields['css_class'].widget = floppyforms.HiddenInput()

    def clean_css_class(self):
        """clean css_class field"""
        instance_fragment_type = self.cleaned_data['type']

        allowed_classes = instance_fragment_type.allowed_css_classes.split(',') if instance_fragment_type else []
        values = self.cleaned_data['css_class']

        if isinstance(values, string_types):
            values = values.strip("[]").split(",")
            values = [x.replace("u'", "").replace('u"', '').strip("\"' ") for x in values]
            values = [x for x in values if x]

        for value in values:
            if not value in allowed_classes:
                raise ValidationError(_("Invalid class '{0}'").format(value))

        return " ".join(values)

    def clean(self):
        ret = super(BaseFragmentForm, self).clean()
        # Check that if no filter is set : the name and type are unique together
        queryset = Fragment.objects.filter(
            name=self.cleaned_data.get('name', ''),
            type=self.cleaned_data.get('type'),
        )
        fragment_filter = self.cleaned_data.get('filter')
        if fragment_filter is None:
            queryset = queryset.filter(filter__isnull=True)
            if self.instance:
                queryset = queryset.exclude(id=self.instance.id)
            if queryset.count():
                raise forms.ValidationError(_(u"A fragment with this name already exists"))
        return ret


class AddFragmentForm(BaseFragmentForm):
    """Add fragment"""
    class Meta:
        model = Fragment
        fields = ('type', 'name', 'css_class', 'position', 'filter', )
        widgets = {
            "filter": forms.HiddenInput(),
        }

    def __init__(self, data=None, *args, **kwargs):
        super(AddFragmentForm, self).__init__(data, *args, **kwargs)  # pylint: disable=E1002
        self.post_init(all_classes=True)


class EditFragmentForm(BaseFragmentForm):
    """Edit fragment"""
    delete_me = floppyforms.BooleanField(label=_("delete"), required=False)

    class Meta:
        model = Fragment
        fields = ('type', 'filter', 'name', 'css_class', 'position', )
        widgets = {
            "type": forms.HiddenInput(),
            "filter": forms.HiddenInput(),
        }
        
    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)
        if instance:
            instance.css_class = instance.css_class.split(" ")
        super(EditFragmentForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        self.post_init()

    def save(self, *args, **kwargs):
        """delete """
        if self.cleaned_data['delete_me']:
            self.instance.delete()
            return None
        return super(EditFragmentForm, self).save(*args, **kwargs)  # pylint: disable=E1002



# -*- coding: utf-8 -*-
"""utils"""

from __future__ import unicode_literals

from threading import current_thread

from coop_cms.moves import MiddlewareMixin


class RequestNotFound(Exception):
    """exception"""
    pass


class RequestManager(object):
    """get django request from anywhere"""
    _shared = {}

    def __init__(self):
        """his is a Borg"""
        self.__dict__ = RequestManager._shared

    def _get_request_dict(self):
        """request dict"""
        if not hasattr(self, '_request'):
            self._request = {}  # pylint: disable=attribute-defined-outside-init
        return self._request

    def clean(self):
        """clean"""
        if hasattr(self, '_request'):
            del self._request

    def get_request(self):
        """return request"""
        _requests = self._get_request_dict()
        the_thread = current_thread()
        if the_thread not in _requests:
            raise RequestNotFound("Request not found: make sure that middleware is installed")
        return _requests[the_thread]

    def set_request(self, request):
        """set request"""
        _requests = self._get_request_dict()
        _requests[current_thread()] = request


class RequestMiddleware(MiddlewareMixin):
    """middleware for request"""

    def process_request(self, request):
        """middleware is called before every request"""
        RequestManager().set_request(request)

    def process_response(self, request, response):
        """process response"""
        # Clear the request : avoid error in unit tests
        RequestManager().set_request(None)
        return response



# -*- coding: utf-8 -*-
"""
coop_bootstrap : bootstrap integration in coop_cms
"""

from __future__ import unicode_literals

default_app_config = 'coop_cms.apps.coop_bootstrap.apps.CoopBootstrapAppConfig'



# -*- coding: utf-8 -*-
"""utils"""

from __future__ import unicode_literals

from django.template.loader import get_template


def get_model_name(model_class):
    """return model name"""
    meta_class = getattr(model_class, '_meta')
    return getattr(meta_class, 'module_name', '') or getattr(meta_class, 'model_name')


def get_model_label(model_class):
    """return model name"""
    meta_class = getattr(model_class, '_meta')
    return getattr(meta_class, 'verbose_name')


def get_model_app(model_class):
    """return app name for this model"""
    meta_class = getattr(model_class, '_meta')
    return getattr(meta_class, 'app_label')


def get_text_from_template(template_name, extra_context=None):
    """
    load a template and render it as text
    :parameter template_name: the path to a template
    :parameter extra_context: some context for rendering
    :return text
    """
    context = extra_context or {}
    template = get_template(template_name)
    return template.render(context)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import coop_cms.settings
import coop_cms.models
import django.utils.timezone
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Alias',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('path', models.CharField(max_length=200)),
                ('redirect_url', models.CharField(default=b'', max_length=200, blank=True)),
            ],
            options={
                'verbose_name': 'Alias',
                'verbose_name_plural': 'Aliases',
            },
        ),
        migrations.CreateModel(
            name='ArticleCategory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('slug', django_extensions.db.fields.AutoSlugField(editable=False, populate_from='name', max_length=100, blank=True, unique=True)),
                ('ordering', models.IntegerField(default=0, verbose_name='ordering')),
                ('in_rss', models.BooleanField(default=False, help_text='The articles of this category will be listed in the main rss feed', verbose_name='in rss')),
                ('sites', models.ManyToManyField(default=[1], to='sites.Site', verbose_name='site')),
            ],
            options={
                'verbose_name': 'article category',
                'verbose_name_plural': 'article categories',
            },
        ),
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(default=django.utils.timezone.now, verbose_name='created', editable=False, blank=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(default=django.utils.timezone.now, verbose_name='modified', editable=False, blank=True)),
                ('name', models.CharField(default=b'', max_length=200, verbose_name='name', blank=True)),
                ('ordering', models.IntegerField(default=100, verbose_name='ordering')),
                ('file', models.FileField(upload_to=coop_cms.models.get_doc_folder, verbose_name='file')),
                ('is_private', models.BooleanField(default=False, help_text='Check this if you do not want to publish this document to all users', verbose_name='is private')),
                ('category', models.ForeignKey(default=None, blank=True, to='coop_cms.ArticleCategory', null=True, verbose_name='category')),
            ],
            options={
                'verbose_name': 'document',
                'verbose_name_plural': 'documents',
            },
        ),
        migrations.CreateModel(
            name='Fragment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name', db_index=True)),
                ('css_class', models.CharField(default='', max_length=100, verbose_name='CSS class', blank=True)),
                ('position', models.IntegerField(default=0, verbose_name='position')),
                ('content', models.TextField(default='', verbose_name='content', blank=True)),
            ],
            options={
                'ordering': ('position', 'id'),
                'verbose_name': 'Fragment',
                'verbose_name_plural': 'Fragment',
            },
        ),
        migrations.CreateModel(
            name='FragmentFilter',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('extra_id', models.CharField(max_length=100, verbose_name='extra_id', db_index=True)),
            ],
            options={
                'verbose_name': 'Fragment filter',
                'verbose_name_plural': 'Fragment filters',
            },
        ),
        migrations.CreateModel(
            name='FragmentType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name', db_index=True)),
                ('allowed_css_classes', models.CharField(default=b'', help_text='the css classed proposed when editing a fragment. It must be separated by comas', max_length=200, verbose_name='allowed css classes')),
            ],
            options={
                'verbose_name': 'Fragment type',
                'verbose_name_plural': 'Fragment types',
            },
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(default=django.utils.timezone.now, verbose_name='created', editable=False, blank=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(default=django.utils.timezone.now, verbose_name='modified', editable=False, blank=True)),
                ('name', models.CharField(default=b'', max_length=200, verbose_name='name', blank=True)),
                ('ordering', models.IntegerField(default=100, verbose_name='ordering')),
                ('file', models.ImageField(upload_to=coop_cms.settings.get_img_folder, verbose_name='file')),
            ],
            options={
                'verbose_name': 'image',
                'verbose_name_plural': 'images',
            },
        ),
        migrations.CreateModel(
            name='ImageSize',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('size', models.CharField(max_length=100, verbose_name='size')),
                ('crop', models.CharField(default=b'', max_length=100, verbose_name='crop', blank=True)),
            ],
            options={
                'verbose_name': 'Image size',
                'verbose_name_plural': 'Image sizes',
            },
        ),
        migrations.CreateModel(
            name='Link',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(default=django.utils.timezone.now, verbose_name='created', editable=False, blank=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(default=django.utils.timezone.now, verbose_name='modified', editable=False, blank=True)),
                ('title', models.CharField(default='title', max_length=200, verbose_name='Title')),
                ('url', models.CharField(max_length=200, verbose_name='URL')),
            ],
            options={
                'verbose_name': 'link',
                'verbose_name_plural': 'links',
            },
        ),
        migrations.CreateModel(
            name='MediaFilter',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
            ],
            options={
                'verbose_name': 'media filter',
                'verbose_name_plural': 'media filters',
            },
        ),
        migrations.CreateModel(
            name='NavNode',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(max_length=200, verbose_name='label')),
                ('ordering', models.PositiveIntegerField(default=0, verbose_name='ordering')),
                ('object_id', models.PositiveIntegerField(null=True, verbose_name='object id', blank=True)),
                ('in_navigation', models.BooleanField(default=True, verbose_name='in navigation')),
                ('content_type', models.ForeignKey(verbose_name='content_type', blank=True, to='contenttypes.ContentType', null=True)),
                ('parent', models.ForeignKey(default=0, blank=True, to='coop_cms.NavNode', null=True, verbose_name='parent')),
            ],
            options={
                'verbose_name': 'navigation node',
                'verbose_name_plural': 'navigation nodes',
            },
        ),
        migrations.CreateModel(
            name='NavTree',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(default=b'default', unique=True, max_length=100, verbose_name='name', db_index=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Navigation tree',
                'verbose_name_plural': 'Navigation trees',
            },
        ),
        migrations.CreateModel(
            name='NavType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('search_field', models.CharField(default=b'', max_length=200, verbose_name='search field', blank=True)),
                ('label_rule', models.IntegerField(default=0, verbose_name='How to generate the label', choices=[(0, 'Use object unicode'), (1, 'Use search field'), (2, 'Use get_label')])),
                ('content_type', models.OneToOneField(verbose_name='django model', to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'navigable type',
                'verbose_name_plural': 'navigable types',
            },
        ),
        migrations.CreateModel(
            name='Newsletter',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('subject', models.CharField(default=b'', max_length=200, verbose_name='subject', blank=True)),
                ('content', models.TextField(default=b'<br>', verbose_name='content', blank=True)),
                ('template', models.CharField(default=b'', max_length=200, verbose_name='template', blank=True)),
                ('source_url', models.URLField(default=b'', verbose_name='source url', blank=True)),
                ('is_public', models.BooleanField(default=False, verbose_name='is_public')),
            ],
            options={
                'verbose_name': 'newsletter',
                'verbose_name_plural': 'newsletters',
            },
        ),
        migrations.CreateModel(
            name='NewsletterItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField(verbose_name='object id')),
                ('ordering', models.IntegerField(default=0, verbose_name='ordering')),
                ('content_type', models.ForeignKey(verbose_name='content_type', to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ['ordering'],
                'verbose_name': 'newsletter item',
                'verbose_name_plural': 'newsletter items',
            },
        ),
        migrations.CreateModel(
            name='NewsletterSending',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('scheduling_dt', models.DateTimeField(default=None, null=True, verbose_name='scheduling date', blank=True)),
                ('sending_dt', models.DateTimeField(default=None, null=True, verbose_name='sending date', blank=True)),
                ('newsletter', models.ForeignKey(to='coop_cms.Newsletter')),
            ],
            options={
                'verbose_name': 'newsletter sending',
                'verbose_name_plural': 'newsletter sendings',
            },
        ),
        migrations.CreateModel(
            name='PieceOfHtml',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('div_id', models.CharField(max_length=100, verbose_name='identifier', db_index=True)),
                ('content', models.TextField(default=b'', verbose_name='content', blank=True)),
                ('extra_id', models.CharField(default=b'', max_length=100, verbose_name='extra identifier', db_index=True, blank=True)),
            ],
            options={
                'verbose_name': 'piece of HTML',
                'verbose_name_plural': 'pieces of HTML',
            },
        ),
        migrations.CreateModel(
            name='SiteSettings',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('homepage_url', models.CharField(default=b'', help_text='if set, the homepage will be redirected to the given URL', max_length=256, verbose_name='homepage URL', blank=True)),
                ('sitemap_mode', models.IntegerField(default=1, choices=[(1, 'Only site articles'), (2, 'All articles')])),
                ('site', models.OneToOneField(verbose_name='site settings', to='sites.Site')),
            ],
            options={
                'ordering': ('site__id',),
                'verbose_name': 'Sites settings',
                'verbose_name_plural': 'Site settings',
            },
        ),
        migrations.AddField(
            model_name='newsletter',
            name='items',
            field=models.ManyToManyField(to='coop_cms.NewsletterItem', blank=True),
        ),
        migrations.AddField(
            model_name='newsletter',
            name='site',
            field=models.ForeignKey(default=1, verbose_name='site', to='sites.Site'),
        ),
        migrations.AddField(
            model_name='navtree',
            name='types',
            field=models.ManyToManyField(related_name='coop_cms_navtree_set', to='coop_cms.NavType', blank=True),
        ),
        migrations.AddField(
            model_name='navnode',
            name='tree',
            field=models.ForeignKey(verbose_name='tree', to='coop_cms.NavTree'),
        ),
        migrations.AddField(
            model_name='image',
            name='filters',
            field=models.ManyToManyField(default=None, to='coop_cms.MediaFilter', verbose_name='filters', blank=True),
        ),
        migrations.AddField(
            model_name='image',
            name='size',
            field=models.ForeignKey(default=None, blank=True, to='coop_cms.ImageSize', null=True, verbose_name='size'),
        ),
        migrations.AddField(
            model_name='fragment',
            name='filter',
            field=models.ForeignKey(default=None, blank=True, to='coop_cms.FragmentFilter', null=True, verbose_name='fragment filter'),
        ),
        migrations.AddField(
            model_name='fragment',
            name='type',
            field=models.ForeignKey(verbose_name='fragment type', to='coop_cms.FragmentType'),
        ),
        migrations.AddField(
            model_name='document',
            name='filters',
            field=models.ManyToManyField(default=None, to='coop_cms.MediaFilter', verbose_name='filters', blank=True),
        ),
        migrations.AlterUniqueTogether(
            name='newsletteritem',
            unique_together=set([('content_type', 'object_id')]),
        ),
    ]



# -*- coding: utf-8 -*-
"""media library unit testing"""

from __future__ import unicode_literals

from django.conf import settings

from datetime import datetime
from unittest import skipIf

from PIL import Image as PilImage

from django.core.files import File
from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.template.base import TemplateSyntaxError
from django.test.utils import override_settings

from model_mommy import mommy
if 'photologue' in settings.INSTALLED_APPS:
    from photologue.models import Photo, Gallery

from coop_cms.models import ArticleCategory, Document, Image, ImageSize, MediaFilter
from coop_cms.moves import BytesIO, get_response_json
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseArticleTest, BaseTestCase, BeautifulSoup, MediaBaseTestCase


class ImageUploadTest(MediaBaseTestCase):
    """Image upload tests"""
    
    def test_view_form_no_filters(self):
        """view upload image when no filters"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_copyright = soup.select("input#id_copyright")
        self.assertEqual(1, len(id_copyright))
        id_size = soup.select("input#id_size")
        self.assertEqual(1, len(id_size))
        self.assertEqual("hidden", id_size[0]["type"])
        id_filters = soup.select("input#id_filters")
        self.assertEqual(1, len(id_filters))
        self.assertEqual("hidden", id_filters[0]["type"])
        
    def test_view_form_no_sizes(self):
        """view upload image when no sizes"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_size = soup.select("input#id_size")
        self.assertEqual(1, len(id_size))
        self.assertEqual("hidden", id_size[0]["type"])

    def test_view_form_with_filters(self):
        """view upload image when filters"""
        mommy.make(MediaFilter, name="icons")
        mommy.make(MediaFilter, name="big-images")
        
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_filters = soup.select("select#id_filters option")
        self.assertEqual(2, len(id_filters))
    
    def test_view_form_with_sizes(self):
        """view upload image when sizes configured"""
        size1 = mommy.make(ImageSize, name="icons")
        size2 = mommy.make(ImageSize, name="big-images")
        
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_sizes = soup.select("select#id_size option")
        self.assertEqual(['', str(size1.id), str(size2.id)], [x["value"] for x in id_sizes])
        
    def test_post_form_no_filters(self):
        """upload image when no filters"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': ''
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, None)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_size(self):
        """upload image with size"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        img_size = mommy.make(ImageSize, size="128")
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': '',
            'size': img_size.id,
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, img_size)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_anonymous(self):
        """upload image anonymous user"""
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': ''
        }
        
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(response.status_code, 302)
        next_url = "/accounts/login/?next={0}".format(url)
        self.assertTrue(response['Location'].find(next_url) >= 0)
        
        images = Image.objects.all()
        self.assertEquals(0, images.count())
        
    def test_post_form_not_allowed(self):
        """upload image when user is not allowed"""
        self._log_as_mediamgr()
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': ''
        }
        
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(response.status_code, 403)
        
        images = Image.objects.all()
        self.assertEquals(0, images.count())
        
    def test_post_form_with_filters(self):
        """upload image with filters"""

        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        filter1 = mommy.make(MediaFilter, name="icons")
        mommy.make(MediaFilter, name="big-images")
        filter3 = mommy.make(MediaFilter, name="small-images")
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': [str(f.id) for f in (filter1, filter3)],
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 2)
        self.assertEqual(list(images[0].filters.all().order_by('id')), [filter1, filter3])
        
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_with_filters_no_choice(self):
        """upload image when filters configured and choice is blank"""

        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        mommy.make(MediaFilter, name="icons")
        mommy.make(MediaFilter, name="big-images")
        mommy.make(MediaFilter, name="small-images")
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': [],
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 0)
        
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_with_invalid_size(self):
        """upload image with invalid size"""

        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': [],
            'size': "hhjk",
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(0, images.count())

    def test_post_form(self):
        """upload image with size"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': '',
            'filters': '',
            'size': '',
            'copyright': '',
        }

        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')

        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, 'unittest1')
        self.assertEqual(images[0].copyright, data['copyright'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, None)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())

    def test_post_form_copyright(self):
        """upload image with size"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': '',
            'size': '',
            'copyright': 'copyright me',
        }

        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')

        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].copyright, data['copyright'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, None)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())


class ImageSizeTest(MediaBaseTestCase):
    """Test media library slide"""
    
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="600")
    def test_image_max_width_no_size(self):
        """get image with no size upscale"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 130)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="600")
    def test_image_max_width_size(self):
        """get image with size"""
        size = mommy.make(ImageSize, size="60", _create_files=True)
        image = mommy.make(Image, size=size, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 60)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="60")
    def test_image_max_width_size_no_scale(self):
        """get image with size downscale"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 60)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="coop_cms.tests.dummy_image_width")
    def test_image_max_width_size_lambda(self):
        """get image from function"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 20)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="")
    def test_image_max_width_size_none(self):
        """get image with size setting is empty"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 130)

    def test_image_no_size(self):
        """get image: no size"""
        image = mommy.make(Image, size=None, _create_files=True)
        url = image.get_absolute_url()
        self.assertEqual(url, image.file.url)

    def test_image_size(self):
        """get image: with size"""
        image_size = mommy.make(ImageSize, size="128x128")
        image = mommy.make(Image, size=image_size, _create_files=True)
        url = image.get_absolute_url()
        self.assertNotEqual(url, image.file.url)

    def test_image_wrong_size(self):
        """get image: wrong size"""
        image_size = mommy.make(ImageSize, size="blabla")
        image = mommy.make(Image, size=image_size, _create_files=True)
        url = image.get_absolute_url()
        self.assertEqual(url, image.file.url)

    def test_image_size_crop(self):
        """get image: cropped"""
        image_size = mommy.make(ImageSize, size="128x128", crop="center")
        image = mommy.make(Image, size=image_size, _create_files=True)
        url = image.get_absolute_url()
        self.assertNotEqual(url, image.file.url)


class MediaLibraryTest(MediaBaseTestCase):
    """Test media library slide"""

    def test_show_images_empty(self):
        """show images empty"""
        self._log_as_mediamgr()
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
    def test_show_documents_empty(self):
        """show documents empty"""
        self._log_as_mediamgr()
        url = reverse('coop_cms_media_documents')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
            
    def test_show_media_anonymous(self):
        """show images empty anonymous"""
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        next_url = "/accounts/login/?next={0}".format(url)
        self.assertTrue(response['Location'].find(next_url) >= 0)
        
    def test_show_media_not_staff(self):
        """show images empty user is not a staff member"""
        self._log_as_mediamgr(is_staff=False)
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)
    
    def test_show_images(self):
        """show images"""
        self._log_as_mediamgr()
        mommy.make(Image, _quantity=2, _create_files=True)
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))
        
    def test_show_images_pagination(self):
        """show images with pagination"""
        self._log_as_mediamgr()
        mommy.make(Image, _quantity=16, _create_files=True)
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(12, len(nodes))
    
    def test_show_images_page_2(self):
        """show images page 2"""
        self._log_as_mediamgr()
        mommy.make(Image, _quantity=16, _create_files=True)
        url = reverse('coop_cms_media_images')+"?page=2"
        response = self.client.get(url, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(200, response.status_code)
        data = get_response_json(response)
        soup = BeautifulSoup(data['html'])
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(4, len(nodes))
        
    def test_show_images_media_filter(self):
        """show images with media filter"""
        self._log_as_mediamgr()
        media_filter = mommy.make(MediaFilter)
        images = []
        for i in range(16):
            images.append(mommy.make(Image, created=datetime(2014, 1, 1, 12, i), _create_files=True))
        images.reverse()
        
        images[5].filters.add(media_filter)
        images[15].filters.add(media_filter)
        url = reverse('coop_cms_media_images')+"?page=1&media_filter={0}".format(media_filter.id)
        response = self.client.get(url, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(200, response.status_code)
        data = get_response_json(response)
        soup = BeautifulSoup(data['html'])
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))
        expected = [x.file.url for x in (images[5], images[15])]
        actual = [node["rel"] for node in nodes]
        self.assertEqual(expected, actual)
    
    def test_show_images_media_filter_all(self):
        """show images empty with all media filter"""
        self._log_as_mediamgr()
        media_filter = mommy.make(MediaFilter)
        
        images = []
        for i in range(16):
            images.append(mommy.make(Image, created=datetime(2014, 1, 1, 12, i), _create_files=True))
        images.reverse()
        
        images[5].filters.add(media_filter)
        images[15].filters.add(media_filter)
        url = reverse('coop_cms_media_images')+"?page=1&media_filter={0}".format(0)
        response = self.client.get(url, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(200, response.status_code)
        data = get_response_json(response)
        soup = BeautifulSoup(data['html'])
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(12, len(nodes))
        expected = [x.file.url for x in images[:12]]
        actual = [node["rel"] for node in nodes]
        self.assertEqual(expected, actual)
    
    
class UploadDocTest(MediaBaseTestCase):
    """Download document"""

    def test_upload_public_doc(self):
        """upload public"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        data = {
            'file': self._get_file(),
            'is_private': False,
            'name': 'a test file',
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        public_docs = Document.objects.filter(is_private=False)
        self.assertEquals(1, public_docs.count())
        self.assertEqual(public_docs[0].name, data['name'])
        self.assertEqual(public_docs[0].category, None)
        file_ = public_docs[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file().read())
        
    def test_upload_public_doc_category(self):
        """upload public category"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        cat = mommy.make(ArticleCategory, name="my cat")
        data = {
            'file': self._get_file(),
            'is_private': False,
            'name': 'a test file',
            'category': cat.id,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        public_docs = Document.objects.filter(is_private=False)
        self.assertEquals(1, public_docs.count())
        self.assertEqual(public_docs[0].name, data['name'])
        self.assertEqual(public_docs[0].category, cat)
        file_ = public_docs[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file().read())
        
    def test_upload_doc_missing_fields(self):
        """upload public: missing fields"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        data = {
            'is_private': False,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.content, b'close_popup_and_media_slide')
        self.assertEquals(0, Document.objects.all().count())

    def test_upload_doc_anonymous_user(self):
        """upload: anonymous user"""

        data = {
            'file': self._get_file(),
            'is_private': False,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.content, b'close_popup_and_media_slide')
        self.assertEquals(0, Document.objects.all().count())
        redirect_url = response.redirect_chain[-1][0]
        login_url = reverse('login')
        self.assertTrue(redirect_url.find(login_url) >= 0)
        
    def test_upload_not_allowed(self):
        """upload: not allowed"""

        self._log_as_mediamgr()
        data = {
            'file': self._get_file(),
            'is_private': False,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 403)
        
    def test_upload_private_doc(self):
        """upload private doc"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        data = {
            'file': self._get_file(),
            'is_private': True,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        private_docs = Document.objects.filter(is_private=True)
        self.assertEquals(1, private_docs.count())
        #TODO : on drone.io filename is unittest1_SomeRandom. Why?
        #self.assertNotEqual(private_docs[0].name, 'unittest1')
        self.assertNotEqual(private_docs[0].name, '')
        self.assertEqual(private_docs[0].category, None)
        file_ = private_docs[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file().read())


class DocsInMediaLibTest(MediaBaseTestCase):
    """view documents in media lib"""

    def test_view_docs(self):
        """view documents"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        file1 = File(self._get_file())
        doc1 = mommy.make(Document, is_private=True, file=file1)
        file2 = File(self._get_file())
        doc2 = mommy.make(Document, is_private=False, file=file2)
        
        response = self.client.get(reverse('coop_cms_media_documents'))
        self.assertEqual(response.status_code, 200)
        
        self.assertContains(response, reverse('coop_cms_download_doc', args=[doc1.id]))
        self.assertNotContains(response, doc1.file.url)
        self.assertNotContains(response, reverse('coop_cms_download_doc', args=[doc2.id]))
        self.assertContains(response, doc2.file.url)
        
    def test_view_docs_anonymous(self):
        """view docs anonymous"""
        url = reverse('coop_cms_media_documents')
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        redirect_url = response.redirect_chain[-1][0]
        login_url = reverse('login')
        self.assertTrue(redirect_url.find(login_url) >= 0)
        
    def test_view_docs_not_allowed(self):
        """view docs not allowed"""
        self._log_as_mediamgr(is_staff=False)
        response = self.client.get(reverse('coop_cms_media_documents'), follow=True)
        self.assertEqual(response.status_code, 403)


@skipIf('photologue' not in settings.INSTALLED_APPS, "photologue is not installed")
class PhotologueInMediaLibTest(MediaBaseTestCase):
    """view documents in media lib"""

    def test_view_photos_empty(self):
        """view medialib when no photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(0, len(nodes))

        nodes = soup.select(".media-filters a")
        self.assertEqual(0, len(nodes))

    def test_view_photos_not_empty(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        mommy.make(Photo)
        mommy.make(Photo)

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))

        nodes = soup.select(".media-filters a")
        self.assertEqual(0, len(nodes))

    def test_view_anonymous(self):
        """view docs anonymous"""
        response = self.client.get(reverse('coop_cms_media_photologue'), follow=True)
        self.assertEqual(response.status_code, 200)
        redirect_url = response.redirect_chain[-1][0]
        login_url = reverse('login')
        self.assertTrue(redirect_url.find(login_url) >= 0)

    def test_view_not_allowed(self):
        """view docs not allowed"""
        self._log_as_mediamgr(is_staff=False)
        response = self.client.get(reverse('coop_cms_media_photologue'), follow=True)
        self.assertEqual(response.status_code, 403)

    def test_view_photos_gallery(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        gallery = mommy.make(Gallery)

        photo1 = mommy.make(Photo)
        photo1.galleries.add(gallery)
        photo1.save()

        mommy.make(Photo)

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))

        nodes = soup.select(".media-filters a")
        #All + gallery
        self.assertEqual(2, len(nodes))

    def test_view_photos_gallery_several(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        gallery = mommy.make(Gallery)
        mommy.make(Gallery)
        mommy.make(Gallery)

        photo1 = mommy.make(Photo)
        photo1.galleries.add(gallery)
        photo1.save()

        mommy.make(Photo)

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))

        nodes = soup.select(".media-filters a")
        #All + galleries
        self.assertEqual(4, len(nodes))

    def test_view_photos_only_gallery(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        gallery1 = mommy.make(Gallery)
        gallery2 = mommy.make(Gallery)

        photo1 = mommy.make(Photo)
        photo1.galleries.add(gallery1)
        photo1.save()

        photo2 = mommy.make(Photo)
        photo2.galleries.add(gallery2)
        photo2.save()

        mommy.make(Photo)

        url = reverse('coop_cms_media_photologue')+"?gallery_filter={0}".format(gallery1.id)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(1, len(nodes))
        self.assertEqual(photo1.title, nodes[0]["title"])

        nodes = soup.select(".media-filters a")
        self.assertEqual(3, len(nodes))


class DownloadDocTest(MediaBaseTestCase):
    """Download document"""

    def test_download_public(self):
        """download public doc"""
        #create a public doc
        file_ = File(self._get_file())
        doc = mommy.make(Document, is_private=False, file=file_)
        
        #check the url
        private_url = reverse('coop_cms_download_doc', args=[doc.id])
        self.assertNotEqual(doc.get_download_url(), private_url)
        
        #login and download
        self._log_as_mediamgr()

        response = self.client.get(doc.get_download_url())
        self.assertEqual(response.status_code, 200)

        content = self.get_safe_content(response)
        self.assertEqual(content, self._get_file().read())
        
        #logout and download
        self.client.logout()
        response = self.client.get(doc.get_download_url())
        self.assertEqual(response.status_code, 200)
        content = self.get_safe_content(response)
        self.assertEqual(content, self._get_file().read())
        
    @skipIf('sanza.Profile' in settings.INSTALLED_APPS, "sanza.Profile installed")
    def test_download_private(self):
        """download private doc"""
            
        #create a public doc
        file_ = File(self._get_file())
        cat = mommy.make(ArticleCategory, name="private-doc")
        doc = mommy.make(Document, is_private=True, file=file_, category=cat)

        #check the url
        private_url = reverse('coop_cms_download_doc', args=[doc.id])
        self.assertEqual(doc.get_download_url(), private_url)
        
        #login and download
        self._log_as_mediamgr()
        response = self.client.get(doc.get_download_url())
        self.assertEqual(response.status_code, 200)
        self.assertEquals(response['Content-Disposition'], "attachment; filename=unittest1.txt")
        self.assertEquals(response['Content-Type'], "text/plain")
        #TODO: This change I/O Exception in UnitTest
        #self.assertEqual(response.content, self._get_file().read()) 
        
        #logout and download
        self.client.logout()
        response = self.client.get(doc.get_download_url(), follow=True)
        self.assertEqual(response.status_code, 200)
        redirect_url = response.redirect_chain[-1][0]
        login_url = reverse('login')
        self.assertTrue(redirect_url.find(login_url) >= 0)
        

class ImageListTemplateTagTest(BaseTestCase):
    """Test image List template tag"""

    def test_non_existing_filter(self):
        """test not existing ist"""
        tpl = Template('{% load coop_utils %}{% coop_image_list "test" as image_list %}{{image_list|length}}')
        html = tpl.render(Context({}))
        self.assertEqual(html, "0")

    def test_empty_filter(self):
        """test empty filter"""
        mommy.make(MediaFilter, name="abcd")
        tpl = Template('{% load coop_utils %}{% coop_image_list "abcd" as image_list %}{{image_list|length}}')
        html = tpl.render(Context({}))
        self.assertEqual(html, "0")

    def test_filter_with_images(self):
        """test filter with image"""
        filter_ = mommy.make(MediaFilter, name="abcd")
        mommy.make(Image, filters=[filter_], _create_files=True)
        tpl = Template('{% load coop_utils %}{% coop_image_list "abcd" as image_list %}{{image_list|length}}')
        html = tpl.render(Context({}))
        self.assertEqual(html, "1")

    def test_filter_with_images_var_name(self):
        """test filter: name from variable"""
        filter_ = mommy.make(MediaFilter, name="abcd")
        mommy.make(Image, filters=[filter_], _create_files=True)
        tpl = Template('{% load coop_utils %}{% coop_image_list filter_name as image_list %}{{image_list|length}}')
        html = tpl.render(Context({"filter_name": filter_.name}))
        self.assertEqual(html, "1")

    def test_filter_as_missing(self):
        """test filter: as is missing"""
        filter_ = mommy.make(MediaFilter, name="abcd")
        mommy.make(Image, filters=[filter_], _create_files=True)
        try:
            Template('{% load coop_utils %}{% coop_image_list "abcd" image_list %}{{image_list|length}}')
        except TemplateSyntaxError as msg:
            self.assertEqual(
                "coop_image_list: usage --> {% coop_image_list 'filter_name' as var_name %}",
                '{0}'.format(msg)
            )
        else:
            self.assertEqual("", "No exception")


class ArticleLogoTest(BaseArticleTest):
    """test article logo"""

    def _get_image(self, file_name='unittest1.png'):
        """get image"""
        return self._get_file(file_name)

    def setUp(self):
        """before each test"""
        super(ArticleLogoTest, self).setUp()
        self._default_article_templates = settings.COOP_CMS_ARTICLE_TEMPLATES
        settings.COOP_CMS_ARTICLE_TEMPLATES = (
            ('test/article_with_logo_size.html', 'Article with logo size'),
            ('test/article_with_logo_size_and_crop.html', 'Article with logo size and crop'),
            ('test/article_no_logo_size.html', 'Article no logo size and crop'),
        )
        self._default_logo_size = getattr(settings, 'COOP_CMS_ARTICLE_LOGO_SIZE', None)
        self._default_logo_crop = getattr(settings, 'COOP_CMS_ARTICLE_LOGO_CROP', None)

    def tearDown(self):
        """after each test"""
        super(ArticleLogoTest, self).tearDown()
        #restore
        settings.COOP_CMS_ARTICLE_TEMPLATES = self._default_article_templates
        settings.COOP_CMS_ARTICLE_LOGO_SIZE = self._default_logo_size
        settings.COOP_CMS_ARTICLE_LOGO_CROP = self._default_logo_crop

    def test_view_article_no_image(self, template_index=0, image=False):
        """article no image"""
        article_class = get_article_class()
        article = mommy.make(
            article_class,
            title="This is my article",
            content="<p>This is my <b>content</b></p>",
            template=settings.COOP_CMS_ARTICLE_TEMPLATES[template_index][0]
        )
        if image:
            article.logo = File(self._get_image())
            article.save()

        response = self.client.get(article.get_absolute_url())
        self.assertEqual(response.status_code, 200)

        self.assertContains(response, article.title)
        self.assertContains(response, article.content)

    def test_view_article_no_image_crop(self):
        """test article no image crop"""
        self.test_view_article_no_image(1)

    def test_view_article_image_no_crop(self):
        """test article no image no crop"""
        self.test_view_article_no_image(0, True)

    def test_view_article_image_and_crop(self):
        """test article image and crop"""
        self.test_view_article_no_image(1, True)

    def test_edit_article_no_image(self, template_index=0, image=False, post_image=False):
        """test edit article no image"""
        article_class = get_article_class()
        if image:
            article = mommy.make(
                article_class,
                title="This is my article",
                content="<p>This is my <b>content</b></p>",
                slug="",
                template=settings.COOP_CMS_ARTICLE_TEMPLATES[template_index][0],
                logo=File(self._get_image())
            )
        else:
            article = mommy.make(
                article_class,
                title="This is my article",
                content="<p>This is my <b>content</b></p>",
                slug="",
                template=settings.COOP_CMS_ARTICLE_TEMPLATES[template_index][0]
            )

        self._log_as_editor()

        response = self.client.post(article.get_edit_url(), follow=True)
        self.assertEqual(response.status_code, 200)

        data = {
            'title': 'Title of the article',
            'content': 'The content',
        }
        if post_image:
            data['logo'] = self._get_image('unittest2.png')
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        article = article_class.objects.get(id=article.id)

        self.assertEqual(data['title'], article.title)
        self.assertEqual(data['content'], article.content)

        self.assertContains(response, article.title)
        self.assertContains(response, article.content)

    def test_edit_article_no_image_crop(self):
        """test edit article no image and crop"""
        self.test_edit_article_no_image(1)

    def test_edit_article_image_no_post_no_crop(self):
        """test edit article image no crop"""
        self.test_edit_article_no_image(0, True, False)

    def test_edit_article_image_no_post_crop(self):
        """test edit article image no crop"""
        self.test_edit_article_no_image(1, True, False)

    def test_edit_article_image_post_no_crop(self):
        """test edit article image no crop"""
        self.test_edit_article_no_image(0, True, True)

    def test_edit_article_image_post_crop(self):
        """test edit article image crop"""
        self.test_edit_article_no_image(1, True, True)

    def test_view_article_no_image_template1(self):
        """test view article no image template 1"""
        self.test_view_article_no_image(2, False)

    def test_view_article_no_image_template2(self):
        """test view article no image template 2"""
        self.test_view_article_no_image(2, True)

    def test_view_article_no_image_template3(self):
        """test view article no image template 3"""
        settings.COOP_CMS_ARTICLE_LOGO_SIZE = "x100"
        settings.COOP_CMS_ARTICLE_LOGO_CROP = "top"
        self.test_view_article_no_image(2, True)

    def test_edit_article_no_image_template1(self):
        """test edit article no image template 1"""
        self.test_edit_article_no_image(2, False, False)

    def test_edit_article_no_image_template2(self):
        """test edit article no image template 2"""
        self.test_edit_article_no_image(2, True, False)

    def test_edit_article_no_image_template3(self):
        """test edit article no image template 3"""
        self.test_edit_article_no_image(2, True, True)

    def test_edit_article_no_image_template4(self):
        """test edit article no image template 4"""
        settings.COOP_CMS_ARTICLE_LOGO_SIZE = "x100"
        settings.COOP_CMS_ARTICLE_LOGO_CROP = "top"
        self.test_edit_article_no_image(2, True, True)



# -*- coding: utf-8 -*-
"""
coop_cms is a Content Management System for Django
"""

from __future__ import unicode_literals


VERSION = (1, 3, 7)


def get_version():
    """returns version number"""
    version = '%s.%s.%s' % (VERSION[0], VERSION[1], VERSION[2])
    return version


__version__ = get_version()


default_app_config = 'coop_cms.apps.CoopCmsAppConfig'



# -*- coding: utf-8 -*-
"""this app is meant for unit testing some parts of coop_cms"""

from __future__ import unicode_literals

default_app_config = 'coop_cms.apps.test_app.apps.CoopCmsTestAppConfig'


# -*- coding: utf-8 -*-
"""utils"""

from .emails import send_email, send_newsletter, strip_a_tags, avoid_line_too_long, make_links_absolute
from .i18n import (
    activate_lang, get_language, get_url_in_language, redirect_to_language, make_locale_path, strip_locale_path
)
from .loaders import get_model_app, get_model_label, get_model_name, get_text_from_template
from .pagination import paginate
from .requests import RequestManager, RequestMiddleware, RequestNotFound
from .settings import get_login_url
from .text import slugify, dehtml
